@@ -86,7 +86,10 @@ def _get_os_environ():
                    and not k.startswith('OS_TEST')  # infra CI var
                    and not k.startswith('OS_STD')   # infra CI var
                    ]
-    if not environkeys:
+    # If the only environ key is region name, don't make a cloud, because
+    # it's being used as a cloud selector
+    if not environkeys or (
+            len(environkeys) == 1 and 'OS_REGION_NAME' in environkeys):
         return None
     for k in environkeys:
         newkey = k[3:].lower()
@@ -110,7 +113,7 @@ def _auth_update(old_dict, new_dict):
 class OpenStackConfig(object):
 
     def __init__(self, config_files=None, vendor_files=None,
-                 override_defaults=None):
+                 override_defaults=None, force_ipv4=None):
         self._config_files = config_files or CONFIG_FILES
         self._vendor_files = vendor_files or VENDOR_FILES
 
@@ -132,11 +135,28 @@ class OpenStackConfig(object):
 
         # Grab ipv6 preference settings from env
         client_config = self.cloud_config.get('client', {})
-        self.prefer_ipv6 = get_boolean(
-            os.environ.pop(
-                'OS_PREFER_IPV6', client_config.get(
-                    'prefer_ipv6', client_config.get(
-                        'prefer-ipv6', False))))
+
+        if force_ipv4 is not None:
+            # If it's passed in to the constructor, honor it.
+            self.force_ipv4 = force_ipv4
+        else:
+            # Get the backwards compat value
+            prefer_ipv6 = get_boolean(
+                os.environ.pop(
+                    'OS_PREFER_IPV6', client_config.get(
+                        'prefer_ipv6', client_config.get(
+                            'prefer-ipv6', True))))
+            force_ipv4 = get_boolean(
+                os.environ.pop(
+                    'OS_FORCE_IPV4', client_config.get(
+                        'force_ipv4', client_config.get(
+                            'broken-ipv6', False))))
+
+            self.force_ipv4 = force_ipv4
+            if not prefer_ipv6:
+                # this will only be false if someone set it explicitly
+                # honor their wishes
+                self.force_ipv4 = True
 
         # Next, process environment variables and add them to the mix
         self.envvar_key = os.environ.pop('OS_CLOUD_NAME', 'envvars')
@@ -147,6 +167,9 @@ class OpenStackConfig(object):
                 ' either your environment based cloud, or one of your'
                 ' file-based clouds.'.format(self.config_filename,
                                              self.envvar_key))
+        # Pull out OS_CLOUD so that if it's the only thing set, do not
+        # make an envvars cloud
+        self.default_cloud = os.environ.pop('OS_CLOUD', None)
 
         envvars = _get_os_environ()
         if envvars:
@@ -419,7 +442,9 @@ class OpenStackConfig(object):
         # _other_ things (SSL arg handling, timeout) all work consistently
         if config['auth_type'] in (None, "None", ''):
             config['auth_type'] = 'admin_token'
-            config['auth']['token'] = None
+            # Set to notused rather than None because validate_auth will
+            # strip the value if it's actually python None
+            config['auth']['token'] = 'notused'
         return loading.get_plugin_loader(config['auth_type'])
 
     def _validate_auth_ksc(self, config):
@@ -521,6 +546,9 @@ class OpenStackConfig(object):
             on missing required auth parameters
         """
 
+        if cloud is None and self.default_cloud:
+            cloud = self.default_cloud
+
         if cloud is None and self.envvar_key in self.get_cloud_names():
             cloud = self.envvar_key
 
@@ -575,7 +603,10 @@ class OpenStackConfig(object):
             if hasattr(value, 'format'):
                 config[key] = value.format(**config)
 
-        prefer_ipv6 = config.pop('prefer_ipv6', self.prefer_ipv6)
+        force_ipv4 = config.pop('force_ipv4', self.force_ipv4)
+        prefer_ipv6 = config.pop('prefer_ipv6', True)
+        if not prefer_ipv6:
+            force_ipv4 = True
 
         if cloud is None:
             cloud_name = ''
@@ -584,7 +615,7 @@ class OpenStackConfig(object):
         return cloud_config.CloudConfig(
             name=cloud_name, region=config['region_name'],
             config=self._normalize_keys(config),
-            prefer_ipv6=prefer_ipv6,
+            force_ipv4=force_ipv4,
             auth_plugin=auth_plugin)
 
     @staticmethod
