@@ -106,6 +106,21 @@ class TestConfig(base.TestCase):
         cc = c.get_one_cloud('_test_cloud_hyphenated')
         self.assertEqual('12345', cc.auth['project_id'])
 
+    def test_get_one_cloud_with_hyphenated_kwargs(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        args = {
+            'auth': {
+                'username': 'testuser',
+                'password': 'testpass',
+                'project-id': '12345',
+                'auth-url': 'http://example.com/v2',
+            },
+            'region_name': 'test-region',
+        }
+        cc = c.get_one_cloud(**args)
+        self.assertEqual('http://example.com/v2', cc.auth['auth_url'])
+
     def test_no_environ(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
@@ -121,17 +136,29 @@ class TestConfig(base.TestCase):
         c.get_one_cloud(cloud='defaults', validate=False)
 
     def test_prefer_ipv6_true(self):
-        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
-                                   vendor_files=[self.vendor_yaml])
-        cc = c.get_one_cloud(cloud='_test-cloud_')
-        self.assertTrue(cc.prefer_ipv6)
-
-    def test_prefer_ipv6_false(self):
         c = config.OpenStackConfig(config_files=[self.no_yaml],
                                    vendor_files=[self.no_yaml])
         cc = c.get_one_cloud(cloud='defaults', validate=False)
+        self.assertTrue(cc.prefer_ipv6)
+
+    def test_prefer_ipv6_false(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cc = c.get_one_cloud(cloud='_test-cloud_')
         self.assertFalse(cc.prefer_ipv6)
 
+    def test_force_ipv4_true(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cc = c.get_one_cloud(cloud='_test-cloud_')
+        self.assertTrue(cc.force_ipv4)
+
+    def test_force_ipv4_false(self):
+        c = config.OpenStackConfig(config_files=[self.no_yaml],
+                                   vendor_files=[self.no_yaml])
+        cc = c.get_one_cloud(cloud='defaults', validate=False)
+        self.assertFalse(cc.force_ipv4)
+
     def test_get_one_cloud_auth_merge(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml])
         cc = c.get_one_cloud(cloud='_test-cloud_', auth={'username': 'user'})
@@ -199,7 +226,7 @@ class TestConfigArgparse(base.TestCase):
     def setUp(self):
         super(TestConfigArgparse, self).setUp()
 
-        self.options = argparse.Namespace(
+        self.args = dict(
             auth_url='http://example.com/v2',
             username='user',
             password='password',
@@ -207,6 +234,7 @@ class TestConfigArgparse(base.TestCase):
             region_name='other-test-region',
             snack_type='cookie',
         )
+        self.options = argparse.Namespace(**self.args)
 
     def test_get_one_cloud_argparse(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -221,7 +249,33 @@ class TestConfigArgparse(base.TestCase):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
 
-        cc = c.get_one_cloud(cloud='', argparse=self.options)
+        cc = c.get_one_cloud(argparse=self.options)
+        self.assertIsNone(cc.cloud)
+        self.assertEqual(cc.region_name, 'other-test-region')
+        self.assertEqual(cc.snack_type, 'cookie')
+
+    def test_get_one_cloud_just_kwargs(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+
+        cc = c.get_one_cloud(**self.args)
+        self.assertIsNone(cc.cloud)
+        self.assertEqual(cc.region_name, 'other-test-region')
+        self.assertEqual(cc.snack_type, 'cookie')
+
+    def test_get_one_cloud_dash_kwargs(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+
+        args = {
+            'auth-url': 'http://example.com/v2',
+            'username': 'user',
+            'password': 'password',
+            'project_name': 'project',
+            'region_name': 'other-test-region',
+            'snack_type': 'cookie',
+        }
+        cc = c.get_one_cloud(**args)
         self.assertIsNone(cc.cloud)
         self.assertEqual(cc.region_name, 'other-test-region')
         self.assertEqual(cc.snack_type, 'cookie')
